@@ -15,6 +15,10 @@ from tempfile import NamedTemporaryFile
 import serial
 import serial.tools.list_ports
 
+from .galvo import Galvo
+from .config import config
+
+
 try:
     import cv2
     is_cv2 = True
@@ -29,32 +33,7 @@ except:
     IS_IMSWITCH = False
 
 
-
-
-class galvo(object):
-    def __init__(self, channel=1, frequency=1000, offset=0, amplitude=1/2, clk_div=0):
-        '''
-        defaults:
-            dac->Setup(DAC_CHANNEL_1, 0, 1, 0, 0, 2);
-            dac->Setup(dac_channel, clk_div, frequency, scale, phase, invert);
-      '''
-        self.channel= channel
-        self.frequency = frequency
-        self.offset = offset
-        self.amplitude = amplitude
-        self.clk_div = clk_div
-        self.path = "/dac_act"
-
-    def return_dict(self):
-        dict = {
-        "task":self.path,
-        "dac_channel": self.channel, # 1 or 2
-        "frequency": self.frequency,
-        "offset": self.offset,
-        "amplitude":self.amplitude,
-        "clk_div": self.clk_div
-        }
-        return dict
+    
 
 
 class logger(object):
@@ -101,7 +80,9 @@ class ESP32Client(object):
     steps_last_2 = 0
     steps_last_3 = 0
 
-    def __init__(self, host=None, port=31950, serialport=None, baudrate=115200):
+    BAUDRATE = 115200
+    
+    def __init__(self, host=None, port=31950, serialport=None, baudrate=BAUDRATE):
         '''
         This client connects to the UC2-REST microcontroller that can be found here
         https://github.com/openUC2/UC2-REST
@@ -113,6 +94,8 @@ class ESP32Client(object):
 
         you can send commands through wifi/http or usb/serial
         '''
+        
+        
 
         if IS_IMSWITCH:
             self.__logger = initLogger(self, tryInheritParent=True)
@@ -120,8 +103,11 @@ class ESP32Client(object):
             self.__logger = logger()            
 
         # initialize galvos
-        self.galvo1 = galvo(channel=1)
-        self.galvo2 = galvo(channel=2)
+        self.galvo1 = Galvo(channel=1)
+        self.galvo2 = Galvo(channel=2)
+        
+        # initialize config
+        self.config = config(self)
 
         self.serialport = serialport
         self.baudrate = baudrate
@@ -145,13 +131,17 @@ class ESP32Client(object):
             self.is_connected = False
             if IS_IMSWITCH: self.__logger.error("No ESP32 device is connected - check IP or Serial port!")
 
+        self.pinConfig = self.config.loadConfigDevice()
         self.__logger.debug("We are connected: "+str(self.is_connected))
 
 
-    def initSerial(self,serialport,baudrate):
+    def initSerial(self,serialport,baudrate=None):
         # use client in wired mode
         self.serialport = serialport # e.g.'/dev/cu.SLAB_USBtoUART'
         self.is_serial = True
+        
+        if baudrate is None:
+           baudrate = self.baudrate 
 
         if IS_IMSWITCH: self.__logger.debug(f'Searching for SERIAL devices...')
         self.is_connected = False
@@ -186,6 +176,10 @@ class ESP32Client(object):
                             self.__logger.error(e)
                         self.is_connected = False
 
+
+    def closeSerial(self):
+        self.serialdevice.close()
+        
     def isConnected(self):
         # check if client is connected to the same network
         s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
@@ -254,24 +248,19 @@ class ESP32Client(object):
                 return None
 
         elif self.is_serial:
-            if self.is_connected or isInit:
-
-                try:
-                    payload["task"]
-                except:
-                    payload["task"] = path
-                try:
-                    is_blocking = payload['isblock']
-                except:
-                    is_blocking = True
-                self.writeSerial(payload)
-                #self.__logger.debug(payload)
-                returnmessage = self.readSerial(is_blocking=is_blocking, timeout=timeout)
-                return returnmessage
-            else:
-                # nothing is connected
-                return -1
-
+            try:
+                payload["task"]
+            except:
+                payload["task"] = path
+            try:
+                is_blocking = payload['isblock']
+            except:
+                is_blocking = True
+            self.writeSerial(payload)
+            #self.__logger.debug(payload)
+            returnmessage = self.readSerial(is_blocking=is_blocking, timeout=timeout)
+            return returnmessage
+        
     def writeSerial(self, payload):
         """Write JSON document to serial device"""
         try:
@@ -279,7 +268,10 @@ class ESP32Client(object):
             self.serialdevice.flushOutput()
         except Exception as e:
             self.__logger.error(e)
-            del self.serialdevice
+            try:
+                del self.serialdevice
+            except:
+                pass
             self.is_connected=False
             # attempt to reconnect?
             try:
@@ -308,10 +300,12 @@ class ESP32Client(object):
                     rmessage =  self.serialdevice.readline().decode()
                     #self.__logger.debug(rmessage)
                     returnmessage += rmessage
-                    if rmessage.find("--")==0 or (time.time()-_time0)>timeout:
+                    if rmessage.find("--")==0:
                         break
                 except:
                     pass
+                if (time.time()-_time0)>timeout:
+                    break
             # casting to dict
             try:
                 returnmessage = json.loads(returnmessage.split("--")[0].split("++")[-1])
@@ -321,6 +315,12 @@ class ESP32Client(object):
         return returnmessage
 
 
+    def loadConfig(self):
+        return self.config.loadConfig()
+
+    def setConfig(self,config={}):
+        return self.config.setConfig(config)
+            
     '''################################################################################################################################################
     HIGH-LEVEL Functions that rely on basic REST-API functions
     ################################################################################################################################################'''
@@ -467,6 +467,10 @@ class ESP32Client(object):
         Send an LED array pattern e.g. an RGB Matrix: led_pattern=np.zeros((3,8,8))
         '''
         path = '/ledarr_act'
+        # Make sure LED strip is filled with matrix information
+        if len(led_pattern.shape)<3:
+            led_pattern = np.reshape(led_pattern, (led_pattern.shape[0], int(np.sqrt(led_pattern.shape[1])), int(np.sqrt(led_pattern.shape[1]))))
+        led_pattern[:,1::2, :] = led_pattern[:,1::2, ::-1]
         payload = {
             "red": led_pattern[0,:].flatten().tolist(),
             "green": led_pattern[1,:].flatten().tolist(),
@@ -484,9 +488,9 @@ class ESP32Client(object):
         path = '/ledarr_act'
         payload = {
             "task":path,
-            "red": intensity[0],
-            "green": intensity[1],
-            "blue": intensity[2],
+            "red": int(intensity[0]),
+            "green": int(intensity[1]),
+            "blue": int(intensity[2]),
             "LEDArrMode": "full"
         }
         self.__logger.debug("Setting LED Pattern (full): "+ str(intensity))
@@ -499,33 +503,32 @@ class ESP32Client(object):
         '''
         path = '/ledarr_act'
         payload = {
-            "red": intensity[0],
-            "green": intensity[1],
-            "blue": intensity[2],
+            "red": int(intensity[0]),
+            "green": int(intensity[1]),
+            "blue": int(intensity[2]),
             "LEDArrMode": pattern
         }
         self.__logger.debug("Setting LED Pattern (full): "+ str(intensity))
         r = self.post_json(path, payload, timeout=timeout)
         return r
 
-    def send_LEDMatrix_single(self, indexled=0, intensity=(255,255,255), Nleds=8*8, timeout=1):
+    def send_LEDMatrix_single(self, indexled=0, intensity=(255,255,255), timeout=1):
         '''
         update only a single LED with a colour:  indexled=0, intensity=(255,255,255)
         '''
         path = '/ledarr_act'
         payload = {
-            "red": intensity[0],
-            "green": intensity[1],
-            "blue": intensity[2],
-            "indexled": indexled,
-            "Nleds": Nleds,
+            "red": int(intensity[0]),
+            "green": int(intensity[1]),
+            "blue": int(intensity[2]),
+            "indexled": int(indexled),
             "LEDArrMode": "single"
         }
         self.__logger.debug("Setting LED PAttern: "+str(indexled)+" - "+str(intensity))
         r = self.post_json(path, payload, timeout=timeout)
         return r
 
-    def send_LEDMatrix_multi(self, indexled=(0), intensity=((255,255,255)), Nleds=8*8, timeout=1):
+    def send_LEDMatrix_multi(self, indexled=(0), intensity=((255,255,255)), timeout=1):
         '''
         update a list of individual LEDs with a colour:  led_pattern=(1,2,6,11), intensity=((255,255,255),(125,122,1), ..)
         '''
@@ -535,7 +538,6 @@ class ESP32Client(object):
             "green": intensity[1],
             "blue": intensity[2],
             "indexled": indexled,
-            "Nleds": Nleds,
             "LEDArrMode": "multi"
         }
         self.__logger.debug("Setting LED PAttern: "+str(indexled)+" - "+str(intensity))
@@ -554,27 +556,11 @@ class ESP32Client(object):
         r = self.post_json(path, payload, timeout=timeout)
         return r
 
-    def set_LEDMatrix(self, LED_ARRAY_PIN=1, LED_N_X=8, LED_N_Y=8, timeout=1):
-        '''
-        set information about pinnumber and number of leds
-        '''
-        # TOOD: Not implemented yet
-        path = '/ledarr_set'
-        payload = {
-            "LED_ARRAY_PIN": intensity[0],
-            "LED_N_X": intensity[1],
-            "LED_N_Y": intensity[2],
-            "indexled": indexled,
-            "Nleds": Nleds,
-            "LEDArrMode": "single"
-        }
-        r = self.post_json(path, payload, timeout=timeout)
-        return r
 
 
     '''
     ##############################################################################################################################
-    MOTOR
+    STATE
     ##############################################################################################################################
     '''
     def isControllerMode(self, timeout=1):
@@ -620,7 +606,11 @@ class ESP32Client(object):
             return r["isBusy"]
         except:
             return r
-
+    '''
+    ##############################################################################################################################
+    MOTOR
+    ##############################################################################################################################
+    '''
     def move_forever(self, speed=(0,0,0), is_stop=False, timeout=1):
         path = "/motor_act"
         payload = {
@@ -1042,3 +1032,8 @@ class ESP32Client(object):
             files = {'media': open(iName, 'rb')}
             if self.is_connected:
                 requests.post(self.base_uri + path, files=files)
+
+
+
+#if __name__ == '__main__':
+    
